CREATE SCHEMA dbo04 AUTHORIZATION dbo
GO

CREATE ROLE excel_users AUTHORIZATION [dbo]
GO

GRANT
    DELETE,
    EXECUTE,
    INSERT,
    SELECT,
    UPDATE,
    VIEW DEFINITION
ON SCHEMA::dbo04
    TO excel_users
GO

CREATE USER excel_user FOR LOGIN excel_user WITH DEFAULT_SCHEMA = dbo04
GO

EXEC sp_addrolemember N'excel_users', N'excel_user'
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_PADDING ON
GO

CREATE TABLE [dbo04].[ExcelTest](
    [ID] [int] IDENTITY(1,1) NOT NULL,
    [Float] [float] NULL,
    [Datetime] [datetime] NULL,
    [Nvarchar] [nvarchar](255) NULL
 CONSTRAINT [PK_ExcelTest_dbo04] PRIMARY KEY CLUSTERED
(
    [ID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
GO

SET ANSI_PADDING OFF
GO

SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE TABLE [dbo04].[ExcelTestImport](
    [ImportID] [int] IDENTITY(1,1) NOT NULL,
    [ID] [int] NULL,
    [Float] [float] NULL,
    [Datetime] [datetime] NULL,
    [Nvarchar] [nvarchar](255) NULL,
 CONSTRAINT [PK_ExcelTestImport_dbo04] PRIMARY KEY CLUSTERED
(
    [ImportID] ASC
)WITH (STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF)
)
GO

SET IDENTITY_INSERT [dbo04].[ExcelTest] ON
GO

INSERT [dbo04].[ExcelTest]
    ([ID], [Float], [Datetime], [Nvarchar])
VALUES
    (1, 123.4567, '20191102 01:00:00', N'Hello, SQL Server!')
    , (2, NULL, NULL, N'Hello, Excel!')
GO

SET IDENTITY_INSERT [dbo04].[ExcelTest] OFF
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

-- =============================================
-- Author:      Sergey Vaselenko
-- Create date: 2011-06-02
-- Description: Export CSV file example
-- =============================================

CREATE PROCEDURE [dbo04].[uspExportExcelTestCSV]
AS
BEGIN
    
    SET NOCOUNT ON

    SELECT
        [ID] AS [Id]
        , ISNULL([Float], '') AS [Float]
        , ISNULL(CONVERT(varchar(19), [Datetime], 120), '') AS [Datetime]
        , ISNULL(QUOTENAME([Nvarchar], '"'), '') AS [Nvarchar]
    FROM
        [dbo04].[ExcelTest]

END

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

-- =============================================
-- Author:      Sergey Vaselenko
-- Create date: 2011-06-07
-- Description: Select from ExcelTest table as procedure
-- =============================================

CREATE PROCEDURE [dbo04].[uspExcelTest]
AS
BEGIN
    
    SET NOCOUNT ON

    SELECT
        [ID]
        , [Float]
        , [Datetime]
        , [Nvarchar]
    FROM
        [dbo04].[ExcelTest]

END

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

-- =============================================
-- Author:      Sergey Vaselenko
-- Create date: 2011-06-07
-- Description: Excel Import Initialization
-- =============================================

CREATE PROCEDURE [dbo04].[uspImportExcel_Before]
    WITH EXECUTE AS SELF
AS
BEGIN
    
    SET NOCOUNT ON

    DELETE dbo04.ExcelTestImport

    -- The command is not available in SQL Azure
    -- DBCC CHECKIDENT ("dbo04.ExcelTestImport", RESEED, 0)

END
GO


SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

-- =============================================
-- Author:      Sergey Vaselenko
-- Create date: 2011-06-07
-- Description: Excel Import Finalization
--
-- Import table contains additional ImportID identity field.
-- So, ID in data imported from Excel can be duplicated.
-- Second and higher duplicated IDs get new values.
-- =============================================

CREATE PROCEDURE [dbo04].[uspImportExcel_After]
    WITH EXECUTE AS SELF
AS
BEGIN
    
    SET NOCOUNT ON

    -- Delete source rows if ID not found in import data

    DELETE dbo04.ExcelTest
    FROM
        dbo04.ExcelTest et
        LEFT OUTER JOIN dbo04.ExcelTestImport i ON et.ID = i.ID
    WHERE
        i.ID IS NULL

    -- Set ID = 0 to NULL to get a regular ID

    UPDATE dbo04.ExcelTestImport
    SET
        ID = NULL
    WHERE
        ID = 0
    
    -- Set non-first IDs in import data to NULL

    UPDATE dbo04.ExcelTestImport
    SET
        ID = NULL
    FROM
        dbo04.ExcelTestImport i
        INNER JOIN (
            SELECT ImportID, ID, ROW_NUMBER() OVER(PARTITION BY ID ORDER BY ImportID) AS Row
            FROM dbo04.ExcelTestImport
        ) r ON r.ImportID = i.ImportID
    WHERE
        r.Row > 1
        AND i.ID IS NOT NULL

    -- Update source data from import data

    UPDATE dbo04.ExcelTest
    SET
        [Float] = i.[Float]
        , [Datetime] = i.[Datetime]
        , [Nvarchar] = i.[Nvarchar]
    FROM
        dbo04.ExcelTest et
        INNER JOIN dbo04.ExcelTestImport i ON et.ID = i.ID


    -- Insert new data with filled ID

    SET IDENTITY_INSERT dbo04.ExcelTest ON

    INSERT dbo04.ExcelTest
        ([ID], [Float], [Datetime], [Nvarchar])
    SELECT
        i.[ID], i.[Float], i.[Datetime], i.[Nvarchar]
    FROM
        dbo04.ExcelTest et
        RIGHT OUTER JOIN dbo04.ExcelTestImport i ON et.ID = i.ID
    WHERE
        et.ID IS NULL
        AND i.ID IS NOT NULL

    SET IDENTITY_INSERT dbo04.ExcelTest OFF

    -- Insert new data with empty ID (as in import or as duplicated ID)

    INSERT dbo04.ExcelTest
        ([Float], [Datetime], [Nvarchar])
    SELECT
        i.[Float], i.[Datetime], i.[Nvarchar]
    FROM
        dbo04.ExcelTestImport i
    WHERE
        i.ID IS NULL

END

GO
